@@ -49,24 +49,31 @@ class FE(object):
         @param df:
         @return:
         """
+        if self.cat_feats is None:
+            self.cat_feats = self.infer_cat_feats(df)
+            logger.info(f"Infer cat feats: {self.cat_feats}")
+
+            if self.num_feats is None:
+                self.num_feats = df.columns.difference(self.cat_feats).tolist()
+                logger.info(f"Infer num feats: {self.num_feats}")
+
         if self.cat_feat_combination is None:
             self.cat_feat_combination = self.combination_all(self.cat_feats)
 
         ##############################################################
         # self.calculate_feature_number()
         ##############################################################
-        if n_jobs == 1:
 
-            for keys in tqdm(self.cat_feat_combination, desc='🐢'):
+        bar = tqdm(self.cat_feat_combination, desc='🐢')
+        if n_jobs == 1:
+            for keys in bar:
                 df_ = self.group_calculate(df, keys)
                 df = df.merge(df_, on=keys)
         else:  # 多进程加速
             def func(keys):
                 return self.group_calculate(df, keys), keys
 
-            df2keys = self.cat_feat_combination | xProcessPoolExecutor(func, n_jobs)
-
-            for df_, keys in tqdm(df2keys, desc='🐢'):
+            for df_, keys in bar | xJobs(func, n_jobs):
                 df = df.merge(df_, on=keys)
 
         return df
@@ -81,14 +88,6 @@ class FE(object):
         return df_
 
     def _get_agg_dict(self, keys):
-        if self.cat_feats is None:
-            self.cat_feats = self.infer_cat_feats(df)
-            logger.info(f"Infer cat feats: {self.cat_feats}")
-
-            if self.num_feats is None:
-                self.num_feats = df.columns.difference(self.cat_feats).tolist()
-                logger.info(f"Infer num feats: {self.num_feats}")
-
         num_agg_dict = {feat: self.num_funcs for feat in self.num_feats}
         cat_agg_dict = {feat: self.cat_funcs for feat in self.cat_feats if feat not in keys}
         return {**cat_agg_dict, **num_agg_dict, **self.agg_dict}
@@ -123,16 +122,11 @@ if __name__ == '__main__':
 
     df = pd.concat([df1, df2], 1)
 
-    ft = FE(cat_feat_combination=[['N0'], ['N1'], ['N2']])
-    # ft = FE()
+    # ft = FE(cat_feat_combination=[['N0'], ['N1'], ['N2']])
+    ft = FE()
 
-    df_ = ft.transform(df, n_jobs=2)
+    df_ = ft.transform(df, n_jobs=4)
 
     print(df_.shape)
 
     print(df_.columns)
-    # def func(i):
-    #     time.sleep(i)
-    #     print(i, 'done!')
-    #
-    # range(4) | xProcessPoolExecutor(func, 10) | xlist
